#include <cstdio>
#include <cstring>
#include <climits>
#include <queue>

#define ALPH 26
#define MAX_N 26
#define MAX_L 1000
#define GRAPH_S (MAX_L + MAX_N)
#define GRAPH_T (MAX_L + MAX_N + 1)
#define GRAPH_N (MAX_L + MAX_N + 2)

int vert_cnt = 0;
char text[GRAPH_N];
int edgeto[GRAPH_N][ALPH];
int dist[GRAPH_N];
int prev[GRAPH_N];
int prevchr[GRAPH_N];
int textind[ALPH];
int textlen[ALPH];

void read_input() {
	int n;
	scanf("%d", &n);

	for (int c = 0; c < ALPH; c++)
		textind[c] = -1;

	for (int i = 0; i < n; i++) {
		int c;
		int len;
		char ch;
		scanf(" %c", &ch);
		c = ch - 'a';
		textind[c] = vert_cnt;
		scanf("%s", &text[vert_cnt]);
		len = strlen(&text[vert_cnt]);
		textlen[c] = len;
		vert_cnt += len + 1;
	}
}

int match_len(char *a, char *b) {
	int i = 0;
	while (a[i] == b[i] && a[i])
		i++;
	return i;
}

void run_edges_S() {
	for (int c = 0; c < ALPH; c++)
		edgeto[GRAPH_S][c] = textind[c];
}

void run_edges_T() {
	for (int c = 0; c < ALPH; c++)
		edgeto[GRAPH_T][c] = -1;
}

void run_edges(int v) {
	for (int c = 0; c < ALPH; c++) {
		edgeto[v][c] = -1;
		int w = textind[c];
		if (w == -1 || v == w) continue;
		int m = match_len(&text[v], &text[w]);
		if (!text[v + m] || !text[w + m]) {
			if (text[v + m])
				edgeto[v][c] = v + m;
			else if (text[w + m])
				edgeto[v][c] = w + m;
			else
				edgeto[v][c] = GRAPH_T;
		}
	}
}

void make_graph() {
	run_edges_S();
	run_edges_T();
	for (int v = 0; v < vert_cnt; v++) {
		if (text[v])
			run_edges(v);
	}
}

void get_distances() {
	std::priority_queue<
		std::pair<int, int>,
		std::vector<std::pair<int, int>>,
		std::greater<std::pair<int, int>>
	> queue;
	queue.emplace(0, GRAPH_S);

	dist[GRAPH_S] = 0;
	dist[GRAPH_T] = INT_MAX;
	for (int v = 0; v < vert_cnt; v++) {
		if (text[v])
			dist[v] = INT_MAX;
	}

	while (!queue.empty()) {
		std::pair<int, int> top = queue.top();
		queue.pop();
		int v = top.second;
		for (int c = 0; c < ALPH; c++) {
			int w = edgeto[v][c];
			if (w == -1) continue;
			if (dist[w] > dist[v] + textlen[c]) {
				dist[w] = dist[v] + textlen[c];
				prev[w] = v;
				prevchr[w] = c;
				queue.emplace(dist[w], w);
			}
		}
	}
}

void print_result() {
	if (dist[GRAPH_T] == INT_MAX) {
		puts("NIE");
		return;
	}

	std::vector<int> chrs;
	std::vector<int> chrs_left;
	std::vector<int> chrs_right;
	for (int v = GRAPH_T; v != GRAPH_S; v = prev[v])
		chrs.push_back(prevchr[v]);

	int bal = 0;
	while (!chrs.empty()) {
		int c = chrs.back();
		chrs.pop_back();
		if (bal > 0) {
			chrs_left.push_back(c);
			bal -= textlen[c];
		} else {
			chrs_right.push_back(c);
			bal += textlen[c];
		}
	}

	printf("%d\n", dist[GRAPH_T] / 2);

	for (int c : chrs_left)
		printf("%s", &text[textind[c]]);
	printf("\n");

	for (int c : chrs_left)
		printf("%c", c + 'a');
	printf("\n");

	for (int c : chrs_right)
		printf("%c", c + 'a');
	printf("\n");
}

int main() {
	read_input();
	make_graph();
	get_distances();
	print_result();
  return 0;
}
